#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Saga Inc.
# Distributed under the terms of the GPL License.

from typing import List
from mitosheet.code_chunks.code_chunk import CodeChunk
from mitosheet.sheet_functions.types.utils import is_datetime_dtype, is_string_dtype, is_timedelta_dtype
from mitosheet.transpiler.transpile_utils import column_header_list_to_transpiled_code, column_header_to_transpiled_code
from mitosheet.types import ColumnHeader, ColumnID


class SplitTextToColumnsCodeChunk(CodeChunk):

    def get_display_name(self) -> str:
        return 'Split text to columns'
    
    def get_description_comment(self) -> str:
        sheet_index: int = self.get_param('sheet_index')
        column_id: ColumnID = self.get_param('column_id')
        delimiters: List[str] = self.get_param('delimiters')
        column_header = self.post_state.column_ids.get_column_header_by_id(sheet_index, column_id)
        delimiters_string = (', ').join(map(lambda x: f'"{x}"', delimiters))
        return f'Split {column_header} on {delimiters_string}'

    def get_code(self) -> List[str]:
        sheet_index: int = self.get_param('sheet_index')
        column_id: ColumnID = self.get_param('column_id')
        delimiters: List[str] = self.get_param('delimiters')
        new_column_headers: List[ColumnHeader] = self.get_execution_data('new_column_headers')

        delimiter_string = repr('|'.join(delimiters))
        
        column_header = self.prev_state.column_ids.get_column_header_by_id(sheet_index, column_id)
        transpiled_column_header = column_header_to_transpiled_code(column_header)
        new_transpiled_column_headers = column_header_list_to_transpiled_code(new_column_headers)
        column_idx = self.prev_state.column_ids.get_column_ids(sheet_index).index(column_id)
        df_name = self.post_state.df_names[sheet_index]

        # Split column
        dtype_string = str(self.prev_state.dfs[sheet_index][column_header].dtype)
        if is_string_dtype(dtype_string):
            string_conversion = ''
        elif is_datetime_dtype(dtype_string):
            string_conversion = ".dt.strftime('%Y-%m-%d %X')"
        elif is_timedelta_dtype(dtype_string):
            string_conversion = ".apply(lambda x: str(x))"
        else:
            string_conversion = ".astype('str')"

        split_column_line = f'{df_name}[{new_transpiled_column_headers}] = {df_name}[{transpiled_column_header}]{string_conversion}.str.split({delimiter_string}, -1, expand=True)'

        # Reorder columns 
        reorder_columns_line = f'{df_name} = {df_name}[{df_name}.columns[:{column_idx + 1}].tolist() + {new_transpiled_column_headers} + {df_name}.columns[{column_idx + 1}:-{len(new_column_headers)}].tolist()]'

        return [split_column_line, reorder_columns_line]

    def get_edited_sheet_indexes(self) -> List[int]:
        return [self.get_param('sheet_index')]



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Saga Inc.
# Distributed under the terms of the GPL License.
from typing import List
from mitosheet.code_chunks.code_chunk import CodeChunk
from mitosheet.transpiler.transpile_utils import column_header_list_to_transpiled_code
from mitosheet.types import ColumnID

class MeltCodeChunk(CodeChunk):

    def get_display_name(self) -> str:
        return 'Melt'
    
    def get_description_comment(self) -> str:
        sheet_index: int = self.get_param('sheet_index')
        id_var_column_ids: List[ColumnID] = self.get_param('id_var_column_ids')
        value_var_column_ids: List[ColumnID] = self.get_param('value_var_column_ids')

        df_name = self.post_state.df_names[sheet_index]
        new_df_name = self.post_state.df_names[-1]
        
        return f"Unpivoted {df_name} into {new_df_name}"

    def get_code(self) -> List[str]:
        sheet_index: int = self.get_param('sheet_index')
        id_var_column_ids: List[ColumnID] = self.get_param('id_var_column_ids')
        value_var_column_ids: List[ColumnID] = self.get_param('value_var_column_ids')

        id_vars = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index, id_var_column_ids)
        value_vars = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index, value_var_column_ids)

        value_vars = list(filter(lambda value_var: value_var not in id_vars, value_vars))

        include_value_vars: bool = self.get_execution_data('include_value_vars')

        df_name = self.post_state.df_names[sheet_index]
        new_df_name = self.post_state.df_names[-1]

        param_string = f'id_vars={column_header_list_to_transpiled_code(id_vars)}'
        if include_value_vars:
            param_string += f', value_vars={column_header_list_to_transpiled_code(value_vars)}'

        return [f'{new_df_name} = {df_name}.melt({param_string})']
    
    def get_created_sheet_indexes(self) -> List[int]:
        return [len(self.post_state.dfs) - 1]
    



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Saga Inc.
# Distributed under the terms of the GPL License.
from typing import List
from mitosheet.code_chunks.code_chunk import CodeChunk
from mitosheet.transpiler.transpile_utils import column_header_list_to_transpiled_code, column_header_to_transpiled_code
from mitosheet.types import ColumnHeader, ColumnID

class OneHotEncodingCodeChunk(CodeChunk):

    def get_display_name(self) -> str:
        return 'One Hot Encoding'
    
    def get_description_comment(self) -> str:
        sheet_index: int = self.get_param('sheet_index')
        column_id: ColumnID = self.get_param('column_id')

        column_header = self.post_state.column_ids.get_column_header_by_id(sheet_index, column_id)

        return f'One-hot Encoded {column_header}'
        
    def get_code(self) -> List[str]:
        sheet_index: int = self.get_param('sheet_index')
        column_id: ColumnID = self.get_param('column_id')
        column_header = self.post_state.column_ids.get_column_header_by_id(sheet_index, column_id)
        new_column_headers: List[ColumnHeader] = self.get_execution_data('new_column_headers')
        df_name = self.post_state.df_names[sheet_index]
        column_index = self.post_state.dfs[sheet_index].columns.tolist().index(column_header)

        transpiled_column_header = column_header_to_transpiled_code(column_header)
        new_transpiled_column_headers = column_header_list_to_transpiled_code(new_column_headers)

        encode_line = f'{df_name}[{new_transpiled_column_headers}] = pd.get_dummies({df_name}[{transpiled_column_header}])'
        reorder_columns_line = f'{df_name} = {df_name}[{df_name}.columns[:{column_index + 1}].tolist() + {new_transpiled_column_headers} + {df_name}.columns[{column_index + 1}:-{len(new_column_headers)}].tolist()]'

        return [
            'import pandas as pd',
            encode_line,
            reorder_columns_line   
        ]

    
    def get_edited_sheet_indexes(self) -> List[int]:
        return [self.get_param('sheet_index')] # TODO: return this here!
    